#version 330

layout(std140) uniform EffectUniforms
{
    float opacity;
    float rotation;
    float zoom;
    uint maskRadius;
    vec2 cloudOffset;
    vec2 podOffset;
    vec2 spawnOffset;
    vec2 texSize;
    vec2 resolution;
    uint flags;
    uint flags2;
    float distanceTravelled;
    float padding[3];
} _36;

uniform sampler2D additionalEffectMask;
uniform sampler2D cloudLayerTextureMask;
uniform sampler2D cloudLayerTexture;

in vec2 vUV;
layout(location = 0) out vec4 fragColor;
in vec4 vColor;
uint maskFlags;
uint maskAdvFlag;
uint quadTexFlag;
uint originFlags;
uint anchorFlags;

bool isOriginOnPod()
{
    return originFlags == 1u;
}

bool isEuclidian()
{
    return (maskFlags == 1u) || (maskFlags == 3u);
}

bool isOutside()
{
    return (maskFlags == 3u) || (maskFlags == 4u);
}

bool isMask()
{
    return maskFlags >= 1u;
}

bool isTexture()
{
    return (maskFlags == 5u) || (maskFlags == 6u);
}

bool isAdvancedOpacityMask()
{
    return maskAdvFlag != 0u;
}

bool isQuadTex()
{
    return quadTexFlag != 0u;
}

vec2 randomQuad(vec2 uv)
{
    vec2 rand = floor(uv);
    rand = (((vec2(0.910000026226043701171875, 0.23000000417232513427734375) * log(vec2(1.0) + (rand * rand))) - (vec2(0.17000000178813934326171875, 0.3499999940395355224609375) * rand.yx)) + (vec2(0.23000000417232513427734375, 0.910000026226043701171875) * log(vec2(1.0) + (rand.yx * rand.yx)))) - (vec2(0.3499999940395355224609375, 0.17000000178813934326171875) * rand);
    rand = floor((rand - floor(rand)) * 2.0) * 0.5;
    return ((uv - floor(uv)) * 0.5) + rand;
}

void main()
{
    maskFlags = _36.flags & 63u;
    maskAdvFlag = (_36.flags & 128u) >> uint(7);
    quadTexFlag = (_36.flags & 64u) >> uint(6);
    originFlags = (_36.flags >> uint(8)) & 255u;
    anchorFlags = _36.flags2 & 255u;
    vec2 maskRadiusUnpacked = unpackHalf2x16(_36.maskRadius);
    float maskRadiusMin = maskRadiusUnpacked.x;
    float maskRadiusMax = maskRadiusUnpacked.y;
    vec2 samplingScale = _36.resolution / _36.texSize;
    float transitionProgress = 3.0 * (_36.opacity - 0.300000011920928955078125);
    vec2 pixelShift = vec2(0.5);
    bool rotationOrigin = false;
    vec2 localUV = ((((vUV - pixelShift) * samplingScale) / vec2(32.0 * _36.zoom)) * 32.0) * 0.5;
    vec2 podUV = localUV;
    vec2 worldUV = localUV + _36.podOffset;
    localUV *= mat2(vec2(cos(_36.rotation), -sin(_36.rotation)), vec2(sin(_36.rotation), cos(_36.rotation)));
    localUV += vec2(0.0, -_36.distanceTravelled);
    localUV -= _36.cloudOffset;
    localUV += _36.spawnOffset;
    localUV += _36.podOffset;
    float _266 = localUV.x;
    bool _270 = abs(_266 / 2.0) < 0.00999999977648258209228515625;
    bool _279;
    if (!_270)
    {
        _279 = abs(localUV.y / 2.0) < 0.00999999977648258209228515625;
    }
    else
    {
        _279 = _270;
    }
    if (_279)
    {
        rotationOrigin = true;
    }
    float _284 = localUV.x;
    bool _288 = abs(_284 / 2.0) < 0.004999999888241291046142578125;
    bool _297;
    if (!_288)
    {
        _297 = abs(localUV.y / 2.0) < 0.004999999888241291046142578125;
    }
    else
    {
        _297 = _288;
    }
    if (_297)
    {
        rotationOrigin = true;
    }
    vec2 originUV = localUV;
    bvec2 _307 = bvec2(isOriginOnPod());
    vec2 maskUV = vec2(_307.x ? podUV.x : originUV.x, _307.y ? podUV.y : originUV.y);
    float _311;
    if (isEuclidian())
    {
        _311 = (length(maskUV) - maskRadiusMin) / (maskRadiusMax - maskRadiusMin);
    }
    else
    {
        _311 = max(max(((maskUV.x + 1.0) + maskRadiusMin) - maskRadiusMax, ((maskUV.y + 1.0) + maskRadiusMin) - maskRadiusMax), max(((maskRadiusMin + 1.0) - maskRadiusMax) - maskUV.x, ((maskRadiusMin + 1.0) - maskRadiusMax) - maskUV.y));
    }
    float localDistance = _311;
    localDistance = clamp(localDistance, 0.0, 1.0);
    float localMask = min(1.0, localDistance);
    if (isOutside())
    {
        localMask = 1.0 - localMask;
    }
    if (!isMask())
    {
        localMask = 1.0;
    }
    if (isTexture())
    {
        vec2 effectUV = (maskUV / vec2(maskRadiusMax)) + vec2(0.5);
        vec4 maskTex = texture(additionalEffectMask, effectUV);
        float _390 = effectUV.x;
        bool _391 = _390 >= 0.0;
        bool _397;
        if (_391)
        {
            _397 = effectUV.y >= 0.0;
        }
        else
        {
            _397 = _391;
        }
        bool _403;
        if (_397)
        {
            _403 = effectUV.x <= 1.0;
        }
        else
        {
            _403 = _397;
        }
        bool _409;
        if (_403)
        {
            _409 = effectUV.y <= 1.0;
        }
        else
        {
            _409 = _403;
        }
        bool showTexture = _409;
        float _411;
        if (showTexture)
        {
            _411 = maskTex.x;
        }
        else
        {
            _411 = float(maskFlags == 6u);
        }
        localMask = _411;
    }
    float opacityBonus = isAdvancedOpacityMask() ? localMask : 1.0;
    float intensityMultiplier = isAdvancedOpacityMask() ? 1.0 : localMask;
    vec2 _431;
    if (isQuadTex())
    {
        vec2 param = localUV;
        _431 = randomQuad(param);
    }
    else
    {
        _431 = localUV;
    }
    vec2 texUV = _431;
    vec4 cloudMasks = texture(cloudLayerTextureMask, texUV);
    float cloudIntensity = clamp((2.0 * ((_36.opacity + opacityBonus) - 1.0)) - cloudMasks.x, 0.0, 1.0) * intensityMultiplier;
    vec4 cloudColor = texture(cloudLayerTexture, texUV);
    fragColor = cloudColor * vec4(1.0, 1.0, 1.0, cloudIntensity);
}

